 ;; nnn-slow.lisp

 ;; Copyright 2018  Christopher Howard

 ;; This program is free software: you can redistribute it and/or modify it
 ;; under the terms of the GNU General Public License as published by
 ;; the Free Software Foundation, either version 3 of the License, or
 ;; (at your option) any later version.

 ;; This program is distributed in the hope that it will be useful, but
 ;; WITHOUT ANY WARRANTY; without even the implied warranty of
 ;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 ;; General Public License for more details.

 ;; You should have received a copy of the GNU General Public License
 ;; along with this program.  If not, see <http://www.gnu.org/licenses/>.

(defun buildf32 (gbyte byte2 byte3 lbyte)
  (list lbyte byte3 byte2 gbyte))

(defun f32logbitp (bit f32)
  (cond
    ((< bit 8) (logbitp bit (first f32)))
    ((< bit 16) (logbitp (- bit 8) (second f32)))
    ((< bit 24) (logbitp (- bit 16) (car (cdr (cdr f32)))))
    (t (logbitp (- bit 24) (car (cdr (cdr (cdr f32))))))))

(defun randomline (blocks)
    (let ((l) nil)
      (dotimes (i blocks)
        (let* ((x (random 32767)) ;; 32767 never generated?
               (s (if (= (random 2) 0) 1 -1))
               (sx (* s x)))
          (setq l (cons sx l))))
      l))

(defun drawline (y line)
  (let ((x 0))
    (mapc (lambda (short)
            (dotimes (b 16)
              (if (logbitp b short)
                  (drawpixel x y))
              (incf x))) line)))

(defun getfive (line pixel)
  (let ((lineptr line)
        (fivelst nil)
        (targetpixel pixel)
        (currentpixel 0)
        (lstlen 0))
    (loop
       (if (= lstlen 5)
             (return (reverse fivelst))
             (if lineptr
                 (if (>= targetpixel (+ currentpixel 16))
                     (progn
                       (setq currentpixel (+ currentpixel 16))
                       (setq lineptr (cdr lineptr)))
                     (progn
                       (setq fivelst
                             (cons (logbitp (mod targetpixel 16)
                                           (car lineptr))
                                   fivelst))
                       (incf lstlen)
                       (incf targetpixel)
                       (gc)))
                 (setq lineptr line))))))


(defun bool2short (five)
  (let ((f (lambda (sum factor lst)
             (if lst
                 (f (+ sum (* factor
                              (if (car lst) 1 0)))
                    (* factor 2) (cdr lst))
                 sum))))
    (f 0 1 five)))

(defun nnline (f32 prevline)
  (let ((nextline nil)
        (pixel 0))
    (mapc (lambda (oldshort)
            (let ((newshort 0))
              (dotimes (b 16)
                (let* ((five (getfive prevline pixel))
                       (rulebit (bool2short five))
                       (newbool (f32logbitp rulebit f32))
                       (newbit (if newbool 1 0))
                       (mask (ash newbit b)))
                  (setq newshort (logxor newshort mask))
                  (gc))
                (incf pixel))
              (setq nextline (cons newshort nextline)))) prevline)
    (reverse nextline)))

(defun drawnnn (rule firstline)
  (let ((currline firstline))
    (dotimes (y 320)
      (drawline y currline)
      (setq currline
            (nnline rule currline)))))
